/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View.StressTests
{
    /**
    *   Stress tests Task 6.
    */
    [TestFixture]
    public class StressRenderAll
    {
        /**
        * Dependencies purposely built during setup to have
        * a large number of sub-elements
        */
        Dependency myDependency, linearDependency;

        /**
        * number of levels the test dependency tree will have
        */
        private const int maxNumLevels = 4;

        /**
         * number of levels used to test the renderAll method
         */
        private const int maxLinearLevels = 100;

        /**
        * number of sub-elements for each level of the test dependency
            * this number is even so as to avoid the blank column issue
        */
        private const int maxNumElements = 11;

        /**
        * Test Diagram renderer
        */
        private IDiagramRenderer sampleRenderer;

        /**
        * Template name for the test files
        */
        private const string fileName = "../../test_files/file00";

        /**
        * counter for number of dependants
        */
        private int totalCount;

        [SetUp]
        public void setUp()
        {
            sampleRenderer = new SVGRenderer(fileName);
            //create the sample dependency graph
            Dependency configManager = new Dependency("Config Manager");
            Dependency spellCheck = new Dependency("Spell Check");
            spellCheck.AddDependant(configManager);

            //create the large dependency graph
            totalCount = 1;
            myDependency = new Dependency("largeDependency");
            Queue currentLevel = new Queue();
            currentLevel.Enqueue(myDependency);
            for(int i = 0; i < maxNumLevels; ++i)
            {
                Queue nextLevel = new Queue();
                while(currentLevel.Count > 0)
                {
                    Dependency current = (Dependency)currentLevel.Dequeue();
                    for(int j = 0; j < maxNumElements; ++j)
                    {
                        Dependency d = new Dependency("level " + i + " element " + j);
                        current.AddDependant(d);
                        nextLevel.Enqueue(d);
                        ++totalCount;
                    }
                }
                currentLevel = new Queue(nextLevel);
            }

            linearDependency = new Dependency("linear dependency");
            Dependency parent = linearDependency;
            for(int i = 0; i < maxLinearLevels; ++i)
            {
                Dependency current = new Dependency("level" + i);
                parent.AddDependant(current);
                parent = current;
            }
        }

        [TearDown]
        public void Teardown()
        {
            File.Delete(fileName);
        }

        /**
        * Attempts to render a large dependency graph
        */
        [Test]
        public void TestLargeGraph()
        {
            // Create the Dependency Renderer and add a diagram renderer
            MockRendererAll renderer = new MockRendererAll();
            renderer.AddDiagramRenderer(sampleRenderer);
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.renderAll(myDependency, true);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering all nodes of a graph with {0} levels and {1} elements per level  (total of {2} elements) took {3} ms", maxNumLevels, maxNumElements, totalCount, ts.TotalMilliseconds);
        }

        /// <summary>
        /// Attempts to render a dependency graph made of a long
        /// sequence of elements each having only one descendant
        /// </summary>
        [Test]
        public void testLongLinearGraph()
        {
            // Create the Dependency Renderer and add a diagram renderer
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer(sampleRenderer);
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.RenderAll(linearDependency, true);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering a dependency graph with " + maxLinearLevels + " levels took " + ts.TotalMilliseconds + " ms");
        }
    }
}